package com.eldos;

import SecureBlackbox.Base.SBEncoding;
import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.TElDCAsyncState;
import SecureBlackbox.Base.TElMemoryStream;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.PDF.TElPDFDocument;
import SecureBlackbox.PDF.TElPDFPublicKeySecurityHandler;
import SecureBlackbox.PDF.TSBPDFPublicKeySignatureType;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;

@SuppressWarnings("serial")
public class ResultServlet extends HttpServlet {
	
	private static final String URL = "<a href=\"/dcweb/result?name=%s\">download</a>";

    private Cache cache = Cache.getInstance();

	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String sessionID = req.getParameter("sessionid");
		
		if ((sessionID == null) || (req.getContentLength() == 0)) {
			resp.sendError(400);
			return;
		}
		if ( ! "application/xml".equals(req.getContentType())) {
			resp.sendError(415);
			return;
		}
		
		InputStream is = req.getInputStream();
		byte[] signature = SBUtils.streamToBuffer(is, 0);
		
		TElMemoryStream result = finishSigning(sessionID, signature);
        cache.put(sessionID, result.getBuffer());

        gotoResult(req, resp, "Signed file: " + String.format(URL, sessionID));
	}
	
	public void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String name = req.getParameter("name");
		
		if (name != null) { // return signed file
			byte[] data = cache.get(name);
			if (data == null) {
				gotoResult(req, resp, "No signed file in cache");
				return;
			}
			
			resp.setContentType("application/pdf");
			resp.setHeader("Content-Disposition", "filename=sample-" + name + ".pdf");
			
			resp.getOutputStream().write(data);
			resp.getOutputStream().flush();
			
		} else { // finish signing
			
			String sessionID = req.getParameter("sessionid");
			
			if (sessionID == null) {
				gotoResult(req, resp, "Empty session");
				return;
			}
				
			String data = req.getParameter("data");
			if (data != null) {
				byte[] signature = SBEncoding.base64DecodeArray(data);

				TElMemoryStream result = finishSigning(sessionID, signature);
                cache.put(sessionID, result.getBuffer());
			}

            gotoResult(req, resp, "Signed file: " + String.format(URL, sessionID));
		}
	}
	
	private void gotoResult(HttpServletRequest req, HttpServletResponse resp, String respStr) throws IOException {
		req.setAttribute("labelResult", respStr);
		try {
			req.getRequestDispatcher("/result.jsp").forward(req, resp);
		} catch (ServletException e) {
			resp.sendError(500);
		}
	}

	private TElMemoryStream finishSigning(String sessionID, byte[] signature) {
		TElDCAsyncState state = new TElDCAsyncState();
        TElMemoryStream input = new TElMemoryStream(signature, 0, signature.length);
        state.loadFromStream(input, SBDCXMLEnc.dcxmlEncoding());
        
        TElPDFDocument doc = new TElPDFDocument();
        TElPDFPublicKeySecurityHandler handler = new TElPDFPublicKeySecurityHandler();
        handler.setSignatureType(TSBPDFPublicKeySignatureType.pstPKCS7SHA1);

        byte[] docBin = cache.get(sessionID);
        TElMemoryStream result = new TElMemoryStream(docBin, 0, docBin.length);
        doc.completeAsyncOperation(result, state, handler);
        result.setPosition(0);
        
        return result;
	}
}
