import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.DAVServer.*;
import SecureBlackbox.HTTPCommon.*;
import SecureBlackbox.HTTPServer.*;
import SecureBlackbox.SSLCommon.*;

public class ClientThread {
	private TElWebDAVServer m_webDavServ;
	private TcpClient m_tcpClient;
	private Logger m_logger;
	private ServerThread m_owner;
    private TElStringList m_users;

	private boolean m_disposed;

	public ClientThread(ServerThread owner, TcpClient tcpClient, TElWebDAVCustomLockList lockList,
		TElCustomFileSystemAdapter diskAdapter, TElCustomCertStorage certStorage, TElStringList users,
		boolean sslMode, Logger logger)
	{
		m_tcpClient = tcpClient;
		m_logger = logger;
		m_owner = owner;
        m_users = users;

		m_owner.AddConnection(this);

		m_webDavServ = new TElWebDAVServer();

		m_webDavServ.setSSLMode(sslMode);
		m_webDavServ.setFileSystemAdapter(diskAdapter);
		m_webDavServ.setGlobalLockList(lockList);
		m_webDavServ.setCertStorage(certStorage);
		m_webDavServ.setPublishExclusiveLocks(true);
		m_webDavServ.setPublishSharedLocks(true);

		m_webDavServ.setOnSend(new TSBSendEvent(DoSend));
		m_webDavServ.setOnReceive(new TSBReceiveEvent(DoReceive));
		m_webDavServ.setOnOpenConnection(new TSBOpenConnectionEvent(DoOpenConnection));
		m_webDavServ.setOnCloseConnection(new SecureBlackbox.SSLCommon.TSBCloseConnectionEvent(DoCloseConnection));
		m_webDavServ.setOnSSLError(new TSBErrorEvent(DoError));
		m_webDavServ.setOnRequest(new TSBWebDavRequestEvent(DoRequest));
        
        m_webDavServ.getControl().setAuthBasic(true);
        m_webDavServ.getControl().setOnGetUserPassword(new TSBOnGetUserPasswordEvent(new TSBOnGetUserPasswordEvent.Callback() {
            @Override
            public void tsbOnGetUserPasswordEventCallback(TObject sender, TElHTTPServerRequestParams reqParams, TSBString passwd, TSBBoolean accept) {
                String username = reqParams.getUsername();
                accept.value = (m_users.indexOfName(username) >= 0);
                passwd.value = m_users.value(username);
            }
        }));

		m_webDavServ.open();
	}
	
	TSBWebDavRequestEvent.Callback DoRequest = new TSBWebDavRequestEvent.Callback() {
		
		public void tsbWebDavRequestEventCallback(TObject arg0,
				TElHTTPServerRequestParams Request, TElMemoryStream data) {
			String method = "";

			TSBHTTPMethod m = Request.getMethod();
			
			if (m == TSBHTTPMethod.hmConnect)
				method = "CONNECT";

			if (m == TSBHTTPMethod.hmDelete)
				method = "DELETE";

			if (m == TSBHTTPMethod.hmGet)
				method = "GET";

			if (m == TSBHTTPMethod.hmHead)
				method = "HEAD";

			if (m == TSBHTTPMethod.hmOptions)
				method = "OPTIONS";
			
			if (m == TSBHTTPMethod.hmPost)
				method = "POST";

			if (m == TSBHTTPMethod.hmPut)
				method = "PUT";

			if (m == TSBHTTPMethod.hmTrace)
				method = "TRACE";

			if (m == TSBHTTPMethod.hmCustom)
				method = Request.getCustomMethod().toUpperCase();
			
			m_logger.AddRequestEvent(method, Request.getURL());			
		}
	};
	
	TSBErrorEvent.Callback DoError = new TSBErrorEvent.Callback() {
		
		public void tsbErrorEventCallback(TObject arg0, int Code, boolean Fatal, boolean Remote) {
			m_logger.AddLogEvent("Error [Code: " + Code + ", Fatal: " + Fatal + ", Remote: " + Remote + "]");
		}
	};
	
	SecureBlackbox.SSLCommon.TSBCloseConnectionEvent.Callback DoCloseConnection = new SecureBlackbox.SSLCommon.TSBCloseConnectionEvent.Callback() {
		@Override
		public void tsbCloseConnectionEventCallback(TObject arg0, TSBCloseReason reason) {
			m_logger.AddLogEvent("Connection closed [" + reason + "]");

		}
	}; 
	
	TSBOpenConnectionEvent.Callback DoOpenConnection = new TSBOpenConnectionEvent.Callback() {
		
		public void tsbOpenConnectionEventCallback(TObject arg0) {
			m_logger.AddLogEvent("New connection");
		}
	}; 
	
	TSBReceiveEvent.Callback DoReceive = new TSBReceiveEvent.Callback() {
		
		public void tsbReceiveEventCallback(TObject arg0, byte[] Buffer, int MaxSize, TSBInteger Written) {
			Written.value = 0;
			try
			{
				Written.value = m_tcpClient.read(Buffer, 0, MaxSize);
				if (Written.value < 0)
					Written.value = 0;
			}
			catch (Exception ex)
			{
				Written.value = 0;
			}
		}
	};
	
	TSBSendEvent.Callback DoSend = new TSBSendEvent.Callback() {
		
		public void tsbSendEventCallback(TObject arg0, byte[] Buffer) {
			m_tcpClient.write(Buffer, 0, Buffer.length);
		}
	};
	
	public void Dispose()
	{
		if (! m_disposed)
		{
			m_webDavServ.close(false);
			m_tcpClient.close();

			m_owner.RemoveConnection(this);
			m_disposed = true;
		}
	}

	public void Execute()
	{			
		while ( ! m_disposed)
		{
			try
			{
				if (m_tcpClient.GetDataAvailable())
				{
					m_webDavServ.dataAvailable();
				}
				else
				{
					java.lang.Thread.sleep(50);
				}
			}
			catch(Exception ex)
			{
				break;
			} 
		}

		Dispose();
	}

}
