import SecureBlackbox.Base.*;
import SecureBlackbox.DAVServer.*;

public class ServerThread {
	private Logger m_logger;
	private TcpListener m_tcpListener;
	private java.lang.Thread m_listenThread;

	private TElWebDAVCustomLockList m_lockList;
	private TElCustomFileSystemAdapter m_diskAdapter;
	private TElCustomCertStorage m_certStorage;
    private TElStringList m_users;
	private boolean m_sslMode;
	private int m_port;
	
	private boolean m_disposed;

	private ArrayList m_list;
	private TElSharedResource m_mutex;

	public ServerThread(Logger logger, int Port, boolean SSLMode, TElWebDAVCustomLockList LockList, 
		TElCustomFileSystemAdapter DiskAdapter, TElCustomCertStorage CertStorage, TElStringList users)
	{
		m_logger = logger;
		m_sslMode = SSLMode;
		m_lockList = LockList;
		m_diskAdapter = DiskAdapter;
		m_certStorage = CertStorage;
        m_users = users;
		m_port = Port;

		m_list = new ArrayList();
		m_mutex = new TElSharedResource();

		m_listenThread = new java.lang.Thread(new Runnable() {
			
			public void run() {
				ListenForClients();
			}
		});
		
		m_listenThread.start();
	}

	public void Dispose()
	{
		if (! m_disposed)
		{
			m_tcpListener.stop();
			m_disposed = true;

			while (m_list.size() > 0)
			{
				((ClientThread)m_list.get(0)).Dispose();
			}
		}
	}

	public void AddConnection(ClientThread clientThread)
	{
		m_mutex.waitToWrite();
		try
		{
			m_list.add(clientThread);
		}
		finally
		{
			m_mutex.done();
		}
	}

	public void RemoveConnection(ClientThread clientThread)
	{
		m_mutex.waitToWrite();
		try
		{
			m_list.remove(clientThread);
		}
		finally
		{
			m_mutex.done();
		}
	}

	private void ListenForClients()
	{
		m_tcpListener = new TcpListener("127.0.0.1", m_port, this) {
			
			void started(TElSocket socket) {
				Logger.Log("TcpListener was started.");
			}
			
			void connected(final TElSocket socket, final ServerThread sender) {
					
				java.lang.Thread job = new java.lang.Thread(new Runnable() {
					public void run() {
						TcpClient client = new TcpClient(socket);

						final ClientThread clientThread = new ClientThread(sender, client, m_lockList, m_diskAdapter,
				                m_certStorage, m_users, m_sslMode, m_logger);
												
						clientThread.Execute();						
					}
				});

				job.start();
			}
		};
		
		m_tcpListener.start();
		m_tcpListener.accept();
	}
}
