import SecureBlackbox.Base.ArrayList;
import SecureBlackbox.ZIP.TElTarArchiveDirectoryEntry;
import SecureBlackbox.ZIP.TElTarWriter;

import javax.swing.table.AbstractTableModel;
import java.util.Vector;

public class KeyTableModel extends AbstractTableModel {
	private static final long serialVersionUID = -4060741556725801933L;
	public static final int FILENAME_INDEX   = 0;
    public static final int SIZE_INDEX  = 1;
    
    protected String[] columnNames = {
    		"File name",
    		"File size" };
    
    protected Vector<KeyDataRow> dataVector = new Vector<KeyDataRow>();

 	public KeyTableModel(TElTarWriter elTarWriter, String currentPath) {
 		TElTarArchiveDirectoryEntry rootEntry = elTarWriter.getDirectory();

		if (currentPath.length() > 0)
		{
			rootEntry = rootEntry.entryWithPath(currentPath);
			if (rootEntry == null)
				return;
		}
		
		if (currentPath.length() > 0)
		{
			dataVector.add(new KeyDataRow("..", null));
		}

		for (int i = 0; i < rootEntry.getEntriesCount(); i++)
		{
			TElTarArchiveDirectoryEntry entry = rootEntry.getEntry(i);
			String name = entry.getFileName(); 
			if (entry.getIsDirectory())
			{
				dataVector.add(new KeyDataRow(name, 0, entry));
			} 
			else
			{
				dataVector.add(new KeyDataRow(name, entry.getFileSize(), entry));
			}
		}
 	}

	public String getColumnName(int column) {
        return columnNames[column];
    }

 	public boolean isCellEditable(int row, int col) {
        return false;
    }
 	
 	public Class<?> getColumnClass(int column) {
         switch (column) {
         	 case FILENAME_INDEX:
         		 return String.class;
             case SIZE_INDEX:
                return Integer.class;
             default:
                return Object.class;
         }
     }

     public Object getValueAt(int row, int col) {
         KeyDataRow record = (KeyDataRow)dataVector.get(row);
         switch (col) {
	         case FILENAME_INDEX:
	        	 return record.FileName;
             case SIZE_INDEX:
                return record.Size;
             default:
                return new Object();
         }
     }

     public void setValueAt(Object value, int row, int column) {
    	 KeyDataRow record = (KeyDataRow)dataVector.get(row);
         switch (column) {
	         case FILENAME_INDEX:
	        	 record.FileName = (String)value;
	        	 break;
             case SIZE_INDEX:
                record.Size = (Integer)value;
                break;
             default:
                System.out.println("invalid index");
         }
         fireTableCellUpdated(row, column);
     }

     public int getRowCount() {
         return dataVector.size();
     }

     public int getColumnCount() {
         return columnNames.length;
     }

	public ArrayList getSelectedItems(int[] indexes) {
		ArrayList result = new ArrayList();
		for(int i = 0; i < dataVector.size(); i++){
			if (selectedIndex(i, indexes))
			{
				KeyDataRow row = (KeyDataRow)dataVector.get(i);
				result.add((TElTarArchiveDirectoryEntry)row.Tag);
			}
		}
		return result;
	}

	public void removeSelectedItems(int[] indexes) {
		Vector<KeyDataRow> result = new Vector<KeyDataRow>();
		for(int i = 0; i < dataVector.size(); i++){
			if (selectedIndex(i, indexes))
				fireTableRowsDeleted(i, i);
			else{
				KeyDataRow row = (KeyDataRow)dataVector.get(i);
				result.add(row);				
			}
		}
		dataVector.clear();
		dataVector.addAll(result);
	}

	private boolean selectedIndex(int i, int[] indexes) {
		for (int j = 0; j < indexes.length; j++)
			if (i == indexes[j]) return true;
		return false;
	}

	public TElTarArchiveDirectoryEntry getFirstSelectedItem(int firstRow) {
		return (TElTarArchiveDirectoryEntry)((KeyDataRow)dataVector.get(firstRow)).Tag;
	}
 }